local Tasks = Tasks
local L = Tasks.L
local TasksUtils = TasksUtils
local db

function Tasks:DisplayInit()
  db = Tasks.db
  self:CreateTasksFrame()
end

-- self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
--                                           --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
--                                           tile = true, tileSize = 16, edgeSize = 16, 
--                                           insets = { left = 4, right = 4, top = 4, bottom = 4 }});

function Tasks:CreateTasksFrame()
  local frameSettings = db.global.displayOptions.frameSettings
  self.TasksFrame = CreateFrame("Frame","TasksFrame",UIParent)
  self.TasksFrame:SetMovable(true)
  self.TasksFrame:EnableMouse(not frameSettings.locked)
  self.TasksFrame:SetClampedToScreen(true)
  self.TasksFrame:RegisterForDrag("LeftButton")
  self.TasksFrame:SetScript("OnDragStart", function(frame) frame:StartMoving() end)
  self.TasksFrame:SetScript("OnDragStop", function(frame) frame:StopMovingOrSizing() end)

  self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            });

  self.TasksFrame:SetBackdropColor(0,0,255,0);
  self.TasksFrame:SetHeight(20)
  self.TasksFrame:SetWidth(150)
  self.TasksFrame:SetAlpha(1.0)

  self.TasksFrame.title = self.TasksFrame:CreateFontString("TasksTitle", nil, "GameFontNormalLarge")
  self.TasksFrame.title:SetText("|cff2FE7EBTasks|r")
  self.TasksFrame.title:SetPoint("TOPLEFT", self.TasksFrame, "TOPLEFT", 5, -5)
  self.TasksFrame.title:Show()

  --set a default location. if user moves it, blizzard will save to layout-local.txt and restore automatically
  self.TasksFrame:SetPoint("CENTER", UIParent, "CENTER", 300, 380)

  self:CreateTaskListFrames(self.TasksFrame)
end

function Tasks:CreateTaskListFrames(anchorFrame)
  for listId, list in pairs(db.global.lists) do
    local listHeaderFrame = CreateFrame("Frame","TasksHeaderFrame"..listId,self.TasksFrame, "TasksHeaderTemplate")
    listHeaderFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            });
    
    listHeaderFrame:SetBackdropColor(255,0,0,0);
    --listHeaderFrame:SetHeight(15)
    --listHeaderFrame:SetWidth(150)
    listHeaderFrame:SetAlpha(1.0)
    listHeaderFrame:SetPoint("TOPLEFT", anchorFrame, "BOTTOMLEFT", 0, 0)
    listHeaderFrame.list = list
    listHeaderFrame.children = {}
    listHeaderFrame:Show()
    --listHeaderFrame.text = listHeaderFrame:CreateFontString("TasksHeader"..listId, nil, "GameFontNormal")
    --listHeaderFrame.text:SetText("|cFF76DEAF"..(list.displayOptions.expanded and "- " or "+ ")..list.name.."|r")
    --listHeaderFrame.text:SetPoint("TOPLEFT", listHeaderFrame, "TOPLEFT", 25, 0)
    --listHeaderFrame.text:Show()
    anchorFrame = listHeaderFrame
    for taskId, task in pairs(list.tasks) do
      local taskFrame = CreateFrame("Frame","TasksItemFrame"..taskId,listHeaderFrame, "TasksItemTemplate")
      taskFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                              --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                              tile = true, tileSize = 16, edgeSize = 16, 
                                              });
      
      taskFrame:SetBackdropColor(255,255,0,0);
      --taskFrame:SetHeight(15)
      --taskFrame:SetWidth(150)
      taskFrame:SetAlpha(1.0)
      taskFrame:SetPoint("TOPLEFT", anchorFrame, "BOTTOMLEFT", 0, 0)
      taskFrame.task = task
      taskFrame:Show()
      table.insert(listHeaderFrame.children,taskFrame)
      --taskFrame.text = taskFrame:CreateFontString("TasksItem"..taskId, nil, "ChatFontNormal")
      --taskFrame.text:SetText(task.name)
      --taskFrame.text:SetPoint("TOPLEFT", taskFrame, "TOPLEFT", 40, 0)
      --taskFrame.text:Show()
      anchorFrame = taskFrame
    end
  end
end

function Tasks:TaskItemClicked(checkbox)
  if checkbox:GetChecked() then
    checkbox:GetParent().task.completedChars[self.currentChar].completed = checkbox:GetChecked()
    checkbox:GetParent().task.completedChars[self.currentChar].timestamp = time()
    local list = checkbox:GetParent():GetParent().list
    local allDone = true;
    for _,v in pairs(list.tasks) do
      if not v.completedChars[self.currentChar].completed then
        allDone = false
      end
    end
    if allDone then
      checkbox:GetParent():GetParent().checkbox:SetChecked(true)
    end
  else
    checkbox:GetParent().task.completedChars[self.currentChar] = nil
    _G[checkbox:GetParent():GetParent():GetName().."_CheckButton"]:SetChecked(false)
  end
end

function Tasks:TaskListClicked(checkbox)
  for k,v in ipairs(checkbox:GetParent().children) do
    if checkbox:GetChecked() then
      v.task.completedChars[self.currentChar].completed = checkbox:GetChecked()
      v.task.completedChars[self.currentChar].timestamp = time()
      v.checkbox:SetChecked(true)
    else
      v.task.completedChars[self.currentChar] = nil
      v.checkbox:SetChecked(false)
    end 
  end
end

function Tasks:TaskHeaderClicked(button)
end